# ProcessLDE - Анализатор длины машинных инструкций

## Обзор

ProcessLDE - это специализированный инструмент, который определяет точную длину инструкций архитектуры x86 и x64. Он использует механизмы отладки Windows для анализа того, как процессор интерпретирует машинные инструкции.

Этот инструмент особенно полезен для:
- Обратной разработки (реверс-инжиниринга)
- Анализа бинарного кода
- Создания дизассемблеров
- Разработки отладчиков и других инструментов безопасности
- Изучения принципов кодирования инструкций x86/x64

## Принцип работы

ProcessLDE использует оригинальный подход для определения длины инструкций:

1. Создаёт копию самого себя как отлаживаемый процесс
2. Размещает байты анализируемой инструкции в определённом месте памяти
3. Использует пошаговое выполнение процессора (флаг трассировки)
4. Перехватывает нарушения доступа к памяти для определения момента завершения обработки инструкции
5. Подсчитывает, сколько байтов было выполнено до достижения конца инструкции

В отличие от методов статического анализа, эта техника использует сам процессор для точного определения границ инструкций, что особенно важно для сложных инструкций x86/x64.

## Возможности

- **Точное измерение**: Использует реальный путь выполнения процессора для определения длины
- **Поддержка двух архитектур**: Работает как в 32-битном (x86), так и в 64-битном (x64) режимах
- **Поддержка всех инструкций x86/x64**: Работает с простыми и сложными инструкциями, включая:
  - Базовые инструкции x86 (PUSH, POP, MOV и т.д.)
  - Специфичные для x64 инструкции с REX-префиксами
  - Поддержка расширенных регистров (R8-R15, XMM8-XMM15)
  - Инструкции с префиксами (REP, LOCK)
  - SIMD-инструкции (SSE, MMX)
  - Инструкции со сложными режимами адресации
- **Автономность**: Отсутствие внешних зависимостей (использует `/NODEFAULTLIB`)
- **Тестирование на реальных примерах**: Может анализировать функции из библиотек Windows DLL

## Сборка и запуск

### Требования
- Операционная система Windows
- Visual Studio 2010 или новее
- Windows SDK

### Сборка
1. Откройте решение (Empty.sln) в Visual Studio
2. Выберите нужную платформу:
   - Для 32-бит: выберите конфигурацию "Release|Win32" или "Debug|Win32"
   - Для 64-бит: выберите конфигурацию "Release|x64" или "Debug|x64"
3. Соберите решение (F7 или Build → Build Solution)
4. Исполняемый файл будет создан в соответствующей папке вывода (например, Release или x64\Release)

### Запуск
Просто запустите исполняемый файл напрямую - никаких дополнительных настроек не требуется. Программа отобразит результаты в консольном окне.

### Понимание вывода

Программа выполняет серию тестовых примеров и отображает результаты в таком виде:

```
Test: INT 3 (Breakpoint)
Bytes: CC
Instruction Length: 0x01 (1) bytes
First Instruction: CC
```

Для каждого тестового случая ProcessLDE показывает:
- Название теста
- Анализируемые сырые байты
- Определённую длину первой инструкции
- Байты, составляющие первую инструкцию
- Оставшиеся байты в последовательности

#### Вывод, специфичный для архитектуры

При запуске x64-версии вы увидите дополнительные тестовые примеры для инструкций, специфичных для x64:

```
Test: x64 REX.W + MOV RAX, [RBX+RCX*4+0x10203040]
Bytes: 48 8B 84 8B 40 30 20 10
Instruction Length: 0x08 (8) bytes
First Instruction: 48 8B 84 8B 40 30 20 10
```

Обратите внимание, как инструкции x64 с REX-префиксами (такими как 48h) правильно анализируются и показывают другую длину, чем их аналоги для x86.

## Чем ProcessLDE отличается от других

Большинство анализаторов длины инструкций используют:
- Жёстко закодированные таблицы опкодов и длин
- Сложную логику декодирования, пытающуюся имитировать работу ЦП

ProcessLDE использует другой подход, позволяя самому процессору определять границы инструкций посредством прямого выполнения. Это часто даёт более точные результаты, особенно для сложного кодирования инструкций.

## Технические детали

### Ключевые компоненты

1. **Создание отлаживаемого процесса**:
   - Создаёт отдельный процесс для анализа инструкций
   - Использует флаги `DEBUG_PROCESS | DEBUG_ONLY_THIS_PROCESS`

2. **Стратегия выделения памяти**:
   - Выделяет две страницы: одну исполняемую, одну защищённую
   - Размещает инструкции на границе для вызова контролируемых нарушений доступа

3. **Пошаговое выполнение**:
   - Устанавливает флаг трассировки EFLAGS (TF) для включения пошагового выполнения инструкций
   - Отлавливает и интерпретирует отладочные события

4. **Обработка нарушений доступа**:
   - Когда происходит нарушение доступа в конкретном месте памяти, это указывает на конец инструкции
   - Подсчитывает обработанные байты для определения длины инструкции

5. **Определение архитектуры**:
   - Автоматически определяет, работает ли в режиме x86 или x64
   - Соответствующим образом адаптирует поведение и регистры указателя инструкций (EIP или RIP)

### Примечания по реализации

- Не использует библиотеки времени выполнения C/C++ (`/NODEFAULTLIB`)
- Использует собственную точку входа (`WMain`)
- Минимальные внешние зависимости
- Оптимизирован для небольшого размера кода
- Учитывает архитектуру с помощью условной компиляции

### Реализация для x64

В режиме x64 ProcessLDE:
- Использует RIP вместо EIP для указателя инструкций
- Использует 64-битные адреса памяти
- Правильно обрабатывает REX-префиксы
- Включает тестовые примеры для инструкций, специфичных для x64
- Корректно анализирует кодирование расширенных регистров

## Ограничения

- Работает только на Windows
- Только инструкции x86/x64
- Требует, чтобы анализируемые инструкции могли быть выполнены без ошибок
- Не может анализировать инструкции с побочными эффектами (например, записывающие в память)

## Лицензия

Этот проект предоставляется "как есть" для образовательных и исследовательских целей.
